-- Setup
local loader = require("AdvTiledLoader/Loader")
loader.path = "maps/"
local HUD = require("HUD.lua")
local damageHandler = require("damageHandler")
local mobHandler = require("mobHandler.lua")
global.hitBoxes = false
require("AnAL")

local map = {
  Desert  = loader.load("desert.tmx"),
  Desert2 = loader.load("desert2.tmx"),
  Cave    = loader.load("cave.tmx")
}

local musicList = {
--  "sounds/music/Lancefield_-_Ethereal(nop_mix).ogg",
--  "sounds/music/gurdonark_-_Restless_Sleep.ogg",
  "sounds/music/cdk_-_Look_to_la_Luna_.ogg",
--  "sounds/music/Fireproof_Babies_-_Swim_below_as_Leviathans.ogg",
}

local currentSong = 1

require "TEsound"
TEsound.playLooping(musicList[currentSong], "music")

local currentMap = "Desert"

local layer = map[currentMap].tl["Ground"]

local displayTime = 0 
local displayMax = 2

global.damageList = {
}

-- This is the char we'll be moving around.
local tempChar = require("char.lua")
Char = map[currentMap].ol["Object1"]:newObject("Char", "Entity",0,0,32,64)
for k,v in pairs(tempChar) do 
  if Char[k] then print("ERROR!  Char is using a variable already in use by the mapping code.  This is seriously bad.  The key is " .. k) end
  Char[k] = v 
end 
tempChar = nil

Char.facing = "down"	-- The direction our char is facing
Char.quads = {		-- The frames of the image
  down  = love.graphics.newQuad(1,1,38,62,145,64),
  up    = love.graphics.newQuad(40,1,38,62,145,64),
  right = love.graphics.newQuad(79,1,32,62,145,64),
  left  = love.graphics.newQuad(112,1,32,62,145,64),
}

-- The image of our char
Char.image = love.graphics.newImage("images/characters/wa/knight_stand.png")

function roundNum(val, decimal)
	if (decimal) then
		return math.floor(((val * 10^decimal) + 0.5) / (10^decimal))
	else
		return math.floor(val+0.5)
	end
end

-- Move the char around the tiles
function Char.moveTile(x,y,dt)
  -- Change the facing direction
  if x > 0 then Char.facing = "right"
  elseif x < 0 then Char.facing = "left"
  elseif y > 0 then Char.facing = "down"
  elseif y < 0 then Char.facing = "up" end

  tempTile = map[currentMap].tiles[map[currentMap].tl["testlayer"].tileData[roundNum(Char.tileY,0)][roundNum(Char.tileX,0)]]
  if tempTile ~= nil and tempTile.properties.pushX then 
    x = x + dt * tempTile.properties.pushX
  end
  if tempTile ~= nil and tempTile.properties.pushY then 
    y = y + dt * tempTile.properties.pushY
  end
  
  -- Grab the tile
  if map[currentMap].tl["Ground"].tileData[roundNum(Char.tileY+y,0)] then
    tile = map[currentMap].tiles[map[currentMap].tl["Ground"].tileData[roundNum(Char.tileY+y,0)][roundNum(Char.tileX+x,0)]]
  else
    tile = nil
  end

  if tile ~= nil and tile.properties.suffer then
    local tempDamage = {}
	tempDamage["damageVal"] = tile.properties.suffer
	if tile.properties.damageType then
	  tempDamage["damageType"] = tile.properties.damageType
	else
	  tempDamage["damageType"] = ""
	end
	table.insert(global.damageList, tempDamage)
  end
  
  if tile == nil or tile.properties.obstacle then 
    tempx = x
    x = 0
  end

  if map[currentMap].tl["Ground"].tileData[roundNum(Char.tileY+y,0)] then
    tile = map[currentMap].tiles[map[currentMap].tl["Ground"].tileData[roundNum(Char.tileY+y,0)][roundNum(Char.tileX+x,0)]]
  else
    tile = nil
  end

  if tile == nil or tile.properties.obstacle then 
    x = tempx
    tempy = y
    y = 0
  end

  if map[currentMap].tl["Ground"].tileData[roundNum(Char.tileY+y,0)] then
    tile = map[currentMap].tiles[map[currentMap].tl["Ground"].tileData[roundNum(Char.tileY+y,0)][roundNum(Char.tileX+x,0)]]
  else
    tile = nil
  end

  -- If the tile is a door, push the char through the door!
  if tile ~= nil and tile.properties.transferX and tile.properties.transferY then
    Char.tileX = tile.properties.transferX
    Char.tileY = tile.properties.transferY
    Char:moveTo((Char.tileX-1)*map[currentMap].tileWidth, (Char.tileY-1)*map[currentMap].tileHeight-40) 
    return
  end
	
  -- If the tile doesn't exist or is an obstacle then exit the function
  if tile == nil or tile.properties.obstacle then return end

  -- Otherwise change the char's tile
  Char.tileX =  x + Char.tileX
  Char.tileY =  y + Char.tileY 
  Char:moveTo((Char.tileX-1)*map[currentMap].tileWidth, (Char.tileY-1)*map[currentMap].tileHeight-40) 
end

-- Do this at first to make sure the char is drawn correctly.
Char.moveTile(0,0)
Char.facing = "down"

-- Draw our char. This function is passed to TileSet.drawAfterTile() which calls it passing the
-- x and y value of the bottom left corner of the tile.
function Char.draw()
  for i, damageParticleSystem in pairs(damageHandler.damageType) do	
    love.graphics.draw(damageParticleSystem, math.floor(Char.x), math.floor(Char.y))
  end

  love.graphics.drawq(Char.image, Char.quads[Char.facing], 
                      (Char.tileX-1)*map[currentMap].tileWidth + Char.width*0.5, 
					  Char.tileY*map[currentMap].tileWidth - Char.height*0.5, 0, 1, 1, Char.width*0.5, Char.height*0.5)
					    
  if global.hitBoxes then
    love.graphics.setColor(50,50,50,50)
    love.graphics.rectangle("line", Char.x, Char.y, Char.width, Char.height)
    love.graphics.setColor(25,25,25,200)
    love.graphics.rectangle("fill", Char.x, Char.y, Char.width, Char.height)
    love.graphics.setColor(255,255,255,255)
  end
  
end

-- Our example class
DesertExample = {}

-- Called from love.keypressed()
function DesertExample.keypressed(k)
  if k == "return" then
    mobHandler.addMob(map, currentMap, "greenBlob", Char.tileX - 2, Char.tileY - 2)
  elseif k == 'g' then
    if map[currentMap].filter == nil then
      map[currentMap].filter = love.graphics.newImage("images/layers/cave_sight.png")
	  global.scale = 1.6
	else
      map[currentMap].filter = nil
    end
  elseif k == 'r' then
    if currentMap == "Desert" then
      currentMap = "Desert2"
	elseif currentMap == "Desert2" then
	  currentMap = "Cave"
	else
	  currentMap = "Desert"
	end

    tempChar = Char
    Char = map[currentMap].ol["Object1"]:newObject("Char", "Entity",0,0,32,64)
    for k,v in pairs(tempChar) do
      if Char[k] then else
        Char[k] = v
      end
    end
    DesertExample.reset()
  elseif k == 'h' then
    global.hitBoxes = not global.hitBoxes
  elseif k == 'k' then
    for k,v in pairs(map[currentMap].ol["Object1"].objects) do
      if v.type == "MobEntity" then 
        map[currentMap].ol["Object1"].objects[k] = nil
      end
	end
  elseif k == 'i' then
    TEsound.stop("music")
    currentSong = currentSong + 1
	if currentSong > #musicList then currentSong = 0 end
	if currentSong > 0 then TEsound.playLooping(musicList[currentSong], "music") end
  end
  
end

-- Resets the example
function DesertExample.reset()
  global.tx = -5
  global.ty = -434
  Char.tileX = 13
  Char.tileY = 24
  Char.moveTile(0,0)
  Char.facing = "down"
  displayTime = 0
  
  print("Reseting desert example")
  for k,v in pairs(map[currentMap].ol["Object1"].objects) do
    if v.type == "MobEntity" then 
	  print("Setting mob to nil")
          map[currentMap].ol["Object1"].objects[k] = nil
	end
  end
end

-- Update the display time for the character control instructions
function DesertExample.update(dt)
  TEsound.cleanup() 

  speedBase = 6  -- An arbitrary constant for controlling player speed
  speedBonus = 0
  
  tile = map[currentMap].tiles[map[currentMap].tl["Ground"].tileData[roundNum(Char.tileY,0)][roundNum(Char.tileX,0)]]
  if tile ~= nil and tile.properties.speed then
    speedBonus = tile.properties.speed
  end
  if love.keyboard.isDown("lshift") then speedBonus = speedBonus + 3 end
  
  speed = speedBase + speedBonus
  
  displayTime = displayTime + dt

  dx=0
  dy=0
  if love.keyboard.isDown("w") then dy=-1
  elseif love.keyboard.isDown("s") then dy=1 end
  if love.keyboard.isDown("a") then dx=-1
  elseif love.keyboard.isDown("d") then dx=1 end

  if dx~= 0 and dy~=0 then 
    dx = dx/math.sqrt(2)
    dy = dy/math.sqrt(2)
  end
  
  tempTile = map[currentMap].tiles[map[currentMap].tl["testlayer"].tileData[roundNum(Char.tileY,0)][roundNum(Char.tileX,0)]]
  if dx~=0 or dy~=0 or (tempTile and tempTile.properties.pushX or tempTile and tempTile.properties.pushY) then
    Char.moveTile(dx*dt*speed,dy*dt*speed, dt) end

  if love.keyboard.isDown("e") then else DesertExample.centerCamera() end
  
  if love.keyboard.isDown("return") then mobHandler.addMob(map, currentMap, "greenBlob", Char.tileX - 2, Char.tileY - 2) end
	  
  for k,v in pairs(map[currentMap].ol["Object1"].objects) do
    if v.type == "MobEntity" then 
	  v.update(dt)
	  if v.dead == true then
        table.remove(map[currentMap].ol["Object1"].objects,k)
      end
	end
  end
       
  damageHandler.update(dt)
  
  -- reset damage for next go-round
  global.damageList = {}
end

function DesertExample.centerCamera()
  global.tx = -Char.tileX * 32 + 400/global.scale + Char.width/2
  global.ty = -Char.tileY * 32 + 300/global.scale + Char.height/2
end
  

-- Called from love.draw()
function DesertExample.draw()

	-- Set sprite batches if they are different than the settings.
	map.useSpriteBatch = global.useBatch

	-- Scale and translate the game screen for map drawing
	local ftx, fty = math.floor(global.tx), math.floor(global.ty)
	love.graphics.push()
	love.graphics.scale(global.scale)
	love.graphics.translate(ftx, fty)
	
	-- Limit the draw range 
	if global.limitDrawing then 
		map[currentMap]:autoDrawRange(ftx, fty, global.scale, -100) 
	else 
		map[currentMap]:autoDrawRange(ftx, fty, global.scale, 50) 
	end
	
	-- Queue our char to be drawn after the tile he's on and then draw the map.
	local maxDraw = global.benchmark and 20 or 1
	for i=1,maxDraw do 
		map[currentMap]:draw() 
	end
		
	-- Reset the scale and translation.
	love.graphics.pop()

	-- Display movement instructions for a second
	if displayTime < displayMax then
		love.graphics.setColor(0,0,0,100)
		love.graphics.rectangle("fill",0,198,love.graphics.getWidth(),17)
		love.graphics.setColor(255,255,255,255)
		love.graphics.print("Use WASD to move me!", 330, 200)
	end
	
  if map[currentMap].filter ~= nil then
    love.graphics.draw(map[currentMap].filter)
  end
	  
  -- HUD display
  damageHandler.draw()
  HUD.draw()
end

return DesertExample
