-- Setup

local mobHandler = {}
local imageBlob  = love.graphics.newImage("images/mobs/blob.png")
local imageBlobShot = love.graphics.newImage("images/mobs/blob_shot.png")

function mobHandler.addMob(map, currentMap, name, tileX, tileY)
  local enemy = map[currentMap].ol["Object1"]:newObject("Mob", "MobEntity",0,0,32,32)
  enemy.tileX = tileX
  enemy.tileY = tileY
  enemy.dead = false
  enemy.timeLastShot = love.timer.getTime()
  loadMob(enemy, name, map, currentMap)

  enemy:moveTo((enemy.tileX-1)*map[currentMap].tileWidth, (enemy.tileY-1)*map[currentMap].tileHeight)
  
  enemy:updateDrawInfo() 
  
  function enemy.update(dt)
    if enemy.x and enemy.y and Char.getDist2(enemy) > 300000 then 
	  if name == "greenBlobShot" then enemy.dead = true end
	  return 
	end
  
    enemy.move(dt)
    if enemy.anim then enemy.anim:update(dt) end
	
	-- If the mob is on an intraversable space, kill it
	local tempTile
	if roundNum(enemy.tileX) <= 0 or roundNum(enemy.tileY) <= 0 then 
	  tempTile = nil 
	else
	  tempTile = map[currentMap].tiles[map[currentMap].tl["Ground"].tileData[roundNum(enemy.tileY,0)][roundNum(enemy.tileX,0)]]
	end
	if tempTile == nil or tempTile.properties.obstacle then enemy.dead = true end
	
	if enemy.touchDamage > 0 and Char.CheckCollision(enemy) then
	  local tempDamage = {}
	  tempDamage["damageVal"] = enemy.touchDamage
      tempDamage["damageType"] = enemy.damageType
	  table.insert(global.damageList, tempDamage)
	end
	
	if name == "greenBlob" and love.timer.getTime() > enemy.timeLastShot + 2.5 then
	  mobHandler.addMob(map, currentMap, "greenBlobShot", enemy.tileX, enemy.tileY)
	  enemy.timeLastShot = love.timer.getTime()
	end
  end
  
  function enemy.move(dt)
    x = dt * enemy.speedX
	y = dt * enemy.speedY
  
    -- Grab the tile
    if map[currentMap].tl["Ground"].tileData[roundNum(enemy.tileY+y,0)] then
      tile = map[currentMap].tiles[map[currentMap].tl["Ground"].tileData[roundNum(enemy.tileY+y,0)][roundNum(enemy.tileX+x,0)]]
    else
      tile = nil
    end
  
    -- If the tile doesn't exist or is an obstacle then choose a new direction and exit the function
    if tile == nil or tile.properties.obstacle then 
      enemy.hitsWall()
      return 
    end
  
    -- Otherwise change the enemy's tile
    enemy.tileX =  x + enemy.tileX
    enemy.tileY =  y + enemy.tileY 
    enemy:moveTo((enemy.tileX-1)*map[currentMap].tileWidth, (enemy.tileY-1)*map[currentMap].tileHeight) 
  end
 
  return enemy
end

function loadMob(enemy, name, map, currentMap)
  if name == "greenBlob" then 
      enemy.hitBox = {
	  X1 = 4,
	  X2 = 28,
	  Y1 = 4,
	  Y2 = 28
	  }
    enemy.touchDamage = 0.5
    enemy.damageType = "poison"
	function enemy.hitsWall()
      -- Character moves in a new direction...
      local temp = math.random() * 2 * math.pi
	  enemy.speedX = 0.7*math.cos(temp)
      enemy.speedY = 0.7*math.sin(temp)
    end 
	-- set initial speedX and speedY by calling hitsWall()
	enemy.hitsWall()

    function enemy.draw()
      enemy.anim:draw(math.floor(enemy.x), math.floor(enemy.y))
	  if global.hitBoxes then
	    love.graphics.setColor(60,60,60,150)
        love.graphics.rectangle("line",enemy.x + enemy.hitBox.X1, enemy.y + enemy.hitBox.Y1, enemy.hitBox.X2 - enemy.hitBox.X1,enemy.hitBox.Y2 - enemy.hitBox.Y1)
        love.graphics.setColor(35,225,20,150)
		love.graphics.rectangle("fill",enemy.x + enemy.hitBox.X1, enemy.y + enemy.hitBox.Y1, enemy.hitBox.X2 - enemy.hitBox.X2, enemy.hitBox.Y2 - enemy.hitBox.Y1)
        love.graphics.setColor(255,255,255,255)
      end
    end
	
	enemy.anim = newAnimation(imageBlob, 32, 32, 0.2, 0)
  
  elseif name == "greenBlobShot" then
    enemy.hitBox = {
	  X1 = 2,
	  X2 = 12,
	  Y1 = 2,
	  Y2 = 12
	}    
    enemy.touchDamage = 0.5
    enemy.damageType = "poison"
	-- Have the mob fire in the direction of the character, +/- 0.4 radians
	local angle = math.atan2(Char.tileY - enemy.tileY, Char.tileX - enemy.tileX) + (0.8*(math.random()-0.5))
    enemy.speedX = 6*math.cos(angle)
    enemy.speedY = 6*math.sin(angle)
	
	function enemy.hitsWall()
	  enemy.dead = true
	  return 
	end 
	
	function enemy.draw()
	  love.graphics.draw(imageBlobShot, enemy.x, enemy.y)
	  if global.hitBoxes then
        love.graphics.setColor(60,60,60,150)
        love.graphics.rectangle("line",enemy.x + enemy.hitBox.X1, enemy.y + enemy.hitBox.Y1, enemy.hitBox.X2 - enemy.hitBox.X1,enemy.hitBox.Y2 - enemy.hitBox.Y1)
	    love.graphics.setColor(225,20,20,150)
		love.graphics.rectangle("fill",enemy.x + enemy.hitBox.X1, enemy.y + enemy.hitBox.Y1, enemy.hitBox.X2 - enemy.hitBox.X2, enemy.hitBox.Y2 - enemy.hitBox.Y1)
        love.graphics.setColor(255,255,255,255)
      end
    end
  else
    -- TODO: Set up some default condition in case a bad mob is created
  end
end


return mobHandler
